#ifndef SUPPORT_CLASSES_H
#define SUPPORT_CLASSES_H

/*
Copyright 2005 Ethan McCallum (ethanqm -at- penguinmail.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

or in the file

	LICENSE

which is distributed with this package.

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
Limitations under the License.
*/

#include<AppException.h>
#include<glob-type.h>

// - - - - - - - - - - - - - - - - - - - - 

#include<functional>


/**
\brief miscellaneous appwide utilties

Some of these are borrowed from my Petal project...

This is a struct, not a class, because it's expected to contain just static
functions or functor objects.  For that same reason, the ctor and copy ctor
are locked off and unimplemented.


*/

class CommonUtil {

	public:

	/// functor to delete pointers of any type...
	
	struct CleanPointerObjects2 {
		template< typename T >
		void operator()( const T* object ) const
			throw()
		{

			if( NULL != object ){
				delete( object ) ;
			}

			return ;

		}

	} ; // struct CleanPointerObjects


	template< typename T >
	struct CleanPointerObjects : public std::unary_function< T , void > {

		void operator()( const T* object ) const
			throw()
		{

			if( NULL != object ){
				delete( object ) ;
			}

			return ;

		}

	} ; // struct CleanPointerObjects


	protected:
	// empty

	private:

	/// unimplemented
	CommonUtil() ;

	CommonUtil( const CommonUtil& ) ;

} ; // class CommonUtil


// - - - - - - - - - - - - - - - - - - - - 

#include<string>
#include<iostream>

class RPM {

	private:
	/// file name of package
	std::string file_ ;

	/// internal package name (based on RPM metadata)
	std::string name_ ;

	/// package architecture (based on RPM metadata)
	std::string arch_ ;

	/// package version (based on RPM metadata)
	std::string version_ ;

	/// package release (based on RPM metadata)
	std::string release_ ;

	/// string format of RPM's  "{version}-{release}"
	std::string fullVersion_ ;

	/// is this a source RPM?
	bool isSourcePackage_ ;
	
	RPM( const RPM& rhs ) ; // not implemented

	protected:
	/* empty */

	public:
	RPM( const std::string& inFile ) throw( AppException ) ;
	// ~RPM() throw() { std::cout << "[dtor for RPM: " << getName() << "]" << std::endl ; }

	const std::string& getFile() const throw() ;
	const std::string& getName() const throw() ;
	const std::string& getArch() const throw() ;
	const std::string& getVersion() const throw() ;
	const std::string& getRelease() const throw() ;
	const std::string& getFullVersion() const throw() ;
	const bool isSourcePackage() const throw() ;

} ; // class RPM

// - - - - - - - - - - - - - - - - - - - - 

/**
\brief used to compare two RPMs, by version
*/

#include<functional>

struct RPMLess
	: public std::binary_function< const RPM& , const RPM& , bool>
{

	bool operator()( const RPM* rpmLeft , const RPM* rpmRight ) const throw() ;

} ; // struct PtrLess



// - - - - - - - - - - - - - - - - - - - - 

#include<set>

typedef std::set< const RPM* , RPMLess > RPMList ;

class Product {

	private:
	const std::string id_ ; // name + architecture
	RPMList list_ ;
	const RPM* latest_ ;

	protected:
	/* empty */

	public:
	Product( const std::string& inId ) ;
	~Product() throw() ;

	void push( const RPM* obj ) ;

	const std::string& getId() const throw() ;

	int size() const throw() ;

	bool empty() const throw() ;

	const RPM* latestVersion() const throw() ;

} ; // class Product

// - - - - - - - - - - - - - - - - - - - - 

/*
QQQ -- have operator() return a bool:
the calling function can track total number of errors
and call exit() with an error code, if need be.
*/

extern "C" {
	#include<unistd.h>
	#include<errno.h>
}

#include<iostream>
#include<sstream>
#include<functional>

class Action : std::unary_function< const RPM* , void >{

	public:
	virtual ~Action() throw() {}

	virtual void operator()( const Product& product ) throw() = 0 ;

	static bool createDir( const char* path ) throw() ;

	static std::string basename( const std::string& in ) ;

} ; // class Action


class ListAction : public Action {

	public:
	~ListAction() throw() {}
	void operator()( const Product& product ) throw() ;

} ; // class ListAction

class PrintAction : public Action {

	public:
	~PrintAction() throw() {}
	void operator()( const Product& product ) throw() ;

} ; // class PrintAction

class HardlinkAction : public Action {

	public:
	HardlinkAction( const std::string& path ) ;
	~HardlinkAction() throw() {}
	void operator()( const Product& product ) throw() ;

	protected:
	/* empty */

	private:
	const std::string path_ ;
	std::ostringstream buf_ ;

} ; // class HardlinkAction


class SoftlinkAction : public Action {

	public:
	SoftlinkAction( const std::string& path ) ;
	~SoftlinkAction() throw() {}
	void operator()( const Product& product ) throw() ;

	protected:
	/* empty */

	private:
	const std::string path_ ;
	std::ostringstream buf_ ;

} ; // class SoftlinkAction

/*

	// QQQ -- to implement

class CopyAction : public Action {
	public:
	CopyAction( const std::string& inTarget ) ;
	~CopyAction() throw() {}
	void operator()( const Product& product ) throw() ;
} ; // class CopyAction
*/




// - - - - - - - - - - - - - - - - - - - - 

#include<map>
#include<string>
#include<sstream>
#include<vector>

typedef std::map< std::string , Product > ProductListInternalMap ;

/**
\brief encapsulates the logic of handling Products

Think of it as a std::map<> with some extra intelligence.
*/

class ProductListIterator
	: public std::iterator< Product , Product >
	// : public std::iterator< Product , std::ptrdiff_t >
{

	public:

	typedef ProductListIterator self_type ;
	typedef self_type& self_reference ;
	typedef std::iterator_traits< ProductListIterator >::value_type value_type ;
	typedef std::iterator_traits< ProductListIterator >::reference reference ;
	typedef std::iterator_traits< ProductListIterator >::difference_type difference_type ;
	typedef std::iterator_traits< ProductListIterator >::pointer pointer ;
	// typedef std::iterator_traits< ProductListIterator >::value_type* pointer ;


	/**
	\brief builds default "end" object

	\param stop end of the range; should be the end of the ProductList's internal map
	*/
	explicit ProductListIterator( ProductListInternalMap::iterator stop ) ;

	/**
	\brief beginning or middle of a ProductList map range
	\param start start of range; should be the beginning of the ProductList's internal map
	\param stop end of the range; should be the end of the ProductList's internal map
	*/
	explicit ProductListIterator( ProductListInternalMap::iterator start , ProductListInternalMap::iterator stop ) ; 


	self_reference operator++() ;
	self_type operator++(int) ;

	bool operator==( const self_reference other ) ;
	bool operator!=( const self_reference other ) ;

	reference operator*() ;

	reference operator->() ;


	protected:
	/* empty */


	private:
	ProductListInternalMap::iterator current_ ;
	ProductListInternalMap::iterator stop_ ;

} ; // class ProductListIterator


/**
\brief a pseudo-STL-type container that holds Products

Think of it as an STL map<> with some app-specific tweaks.

For example, a map<> is used to store the underlying Product
objects (such that they are searchable, for when a new RPM
is to be added) but the iterators permit algorithms to operate
on the Product objects directly. Otherwise, algorithms would
have to deal with the specifics of the underlying map<> and call
"map<>::iterator<>::second"...
*/

class ProductList {

	public:
	typedef ProductListIterator iterator ;
	typedef ProductListInternalMap::value_type value_type ;

	ProductList() ;

	void add( const std::string path ) ;
	void add( const RPM* rpm ) ;

	bool empty() const throw() ;
	iterator begin() ;
	iterator end() ;
	int size() const throw() ;


	private:

	ProductListInternalMap map_ ;

} ; // ProductList


// - - - - - - - - - - - - - - - - - - - - 

class RPMSource {

	public:
	virtual ~RPMSource() throw() {} 

	virtual void load( ProductList& pl ) throw( AppException ) = 0 ;

	// QQQ: coming soon...
	// virtual size_t size() const throw() = 0 ;

} ; // class RPMSource


class DirectoryRPMSource : public RPMSource {

	public:
	DirectoryRPMSource( const std::string& dir ) throw( AppException ) ;

	void load( ProductList& pl ) throw( AppException ) ;

	protected:
	// empty

	private:
	GlobObj glob_ ;

} ; // class DirectoryRPMSource

// - - - - - - - - - - - - - - - - - - - - 

#endif // #ifndef SUPPORT_CLASSES_H
